@@ -14,6 +14,7 @@ import logging
 from .embedding import make_mat, make_sprite, make_tsv, append_pbtxt
 from .event_file_writer import EventFileWriter
 from .onnx_graph import load_onnx_graph
+from .openvino_graph import load_openvino_graph
 from .pytorch_graph import graph
 from .proto import event_pb2
 from .proto import summary_pb2
@@ -21,7 +22,7 @@ from .proto.event_pb2 import SessionLog, Event
 from .utils import figure_to_image
 from .summary import (
     scalar, histogram, histogram_raw, image, audio, text,
-    pr_curve, pr_curve_raw, video, custom_scalars, image_boxes, mesh
+    pr_curve, pr_curve_raw, video, custom_scalars, image_boxes, mesh, hparams
 )
 
 
@@ -141,7 +142,7 @@ class FileWriter(object):
         self.add_event(event, None, walltime)
 
         trm = event_pb2.TaggedRunMetadata(
-            tag='step1', run_metadata=stepstats.SerializeToString())
+            tag='profiler', run_metadata=stepstats.SerializeToString())
         event = event_pb2.Event(tagged_run_metadata=trm)
         self.add_event(event, None, walltime)
 
@@ -156,6 +157,17 @@ class FileWriter(object):
         event = event_pb2.Event(graph_def=graph.SerializeToString())
         self.add_event(event, None, walltime)
 
+    def add_openvino_graph(self, graph, walltime=None):
+        """Adds a `Graph` protocol buffer to the event file.
+
+        Args:
+          graph: A `Graph` protocol buffer.
+          walltime: float. Optional walltime to override the default (current)
+            _get_file_writerfrom time.time())
+        """
+        event = event_pb2.Event(graph_def=graph.SerializeToString())
+        self.add_event(event, None, walltime)
+
     def flush(self):
         """Flushes the event file to disk.
         Call this method to make sure that all pending events have been written to
@@ -322,6 +334,47 @@ class SummaryWriter(object):
             self.all_writers = {self.file_writer.get_logdir(): self.file_writer}
         return self.file_writer
 
+    def add_hparams(self, hparam_dict=None, metric_dict=None, name=None, global_step = None):
+        """Add a set of hyperparameters to be compared in tensorboard.
+
+        Args:
+            hparam_dict (dictionary): Each key-value pair in the dictionary is the
+              name of the hyper parameter and it's corresponding value.
+            metric_dict (dictionary): Each key-value pair in the dictionary is the
+              name of the metric and it's corresponding value. Note that the key used
+              here should be unique in the tensorboard record. Otherwise the value
+              you added by `add_scalar` will be displayed in hparam plugin. In most
+              cases, this is unwanted.
+            name (string): Personnalised name of the hparam session
+            global_step (int): Current time step
+
+        Examples::
+
+            from tensorboardX import SummaryWriter
+            with SummaryWriter() as w:
+                for i in range(5):
+                    w.add_hparams({'lr': 0.1*i, 'bsize': i},
+                                  {'hparam/accuracy': 10*i, 'hparam/loss': 10*i})
+
+        Expected result:
+
+        .. image:: _static/img/tensorboard/add_hparam.png
+           :scale: 50 %
+        """
+        if type(hparam_dict) is not dict or type(metric_dict) is not dict:
+            raise TypeError('hparam_dict and metric_dict should be dictionary.')
+        exp, ssi, sei = hparams(hparam_dict, metric_dict)
+
+        if not name:
+            name = str(time.time())
+            
+        with SummaryWriter(logdir=os.path.join(self.file_writer.get_logdir(), name)) as w_hp:
+            w_hp.file_writer.add_summary(exp)
+            w_hp.file_writer.add_summary(ssi)
+            w_hp.file_writer.add_summary(sei)
+            for k, v in metric_dict.items():
+                w_hp.add_scalar(k, v, global_step)
+
     def add_scalar(self, tag, scalar_value, global_step=None, walltime=None):
         """Add scalar data to summary.
 
@@ -616,7 +669,7 @@ class SummaryWriter(object):
             walltime (float): Optional override default walltime (time.time()) of event
             labels (list of string): The strings to be show on each bounding box.
         Shape:
-            img_tensor: Default is :math:`(3, H, W)`. It can be specified with ``dataformat`` agrument.
+            img_tensor: Default is :math:`(3, H, W)`. It can be specified with ``dataformats`` argument.
             e.g. CHW or HWC
 
             box_tensor: (torch.Tensor, numpy.array, or string/blobname): NX4,  where N is the number of
@@ -703,10 +756,23 @@ class SummaryWriter(object):
         self._get_file_writer().add_summary(
             text(tag, text_string), global_step, walltime)
 
-    def add_onnx_graph(self, prototxt):
-        self._get_file_writer().add_onnx_graph(load_onnx_graph(prototxt))
+    def add_onnx_graph(self, onnx_model_file):
+        """Add onnx graph to TensorBoard.
 
-    def add_graph(self, model, input_to_model=None, verbose=False, **kwargs):
+        Args:
+            onnx_model_file (string): The path to the onnx model.
+        """
+        self._get_file_writer().add_onnx_graph(load_onnx_graph(onnx_model_file))
+
+    def add_openvino_graph(self, xmlname):
+        """Add openvino graph to TensorBoard.
+
+        Args:
+            xmlname (string): The path to the openvino model. (the xml file)
+        """
+        self._get_file_writer().add_openvino_graph(load_openvino_graph(xmlname))
+
+    def add_graph(self, model, input_to_model=None, verbose=False, profile_with_cuda=False, **kwargs):
         # prohibit second call?
         # no, let tensorboard handle it and show its warning message.
         """Add graph data to summary.
@@ -735,7 +801,7 @@ class SummaryWriter(object):
                 if not hasattr(torch.autograd.Variable, 'grad_fn'):
                     print('add_graph() only supports PyTorch v0.2.')
                     return
-            self._get_file_writer().add_graph(graph(model, input_to_model, verbose, **kwargs))
+            self._get_file_writer().add_graph(graph(model, input_to_model, verbose, profile_with_cuda, **kwargs))
         else:
             # Caffe2 models do not have the 'forward' method
             from caffe2.proto import caffe2_pb2
@@ -773,13 +839,18 @@ class SummaryWriter(object):
         Args:
             mat (torch.Tensor or numpy.array): A matrix which each row is the feature vector of the data point
             metadata (list): A list of labels, each element will be convert to string
-            label_img (torch.Tensor): Images correspond to each data point
+            label_img (torch.Tensor or numpy.array): Images correspond to each data point. Each image should
+                be square. The amount and size of the images are limited by the Tensorboard frontend,
+                see limits below.
             global_step (int): Global step value to record
             tag (string): Name for the embedding
         Shape:
             mat: :math:`(N, D)`, where N is number of data and D is feature dimension
 
-            label_img: :math:`(N, C, H, W)`
+            label_img: :math:`(N, C, H, W)`, where `Height` should be equal to `Width`.
+            Also, :math:`\sqrt{N}*W` must be less than or equal to 8192, so that the generated sprite
+            image can be loaded by the Tensorboard frontend
+            (see `tensorboardX#516 <https://github.com/lanpa/tensorboardX/issues/516>`_ for more).
 
         Examples::
 
@@ -793,7 +864,7 @@ class SummaryWriter(object):
             for i, v in enumerate(meta):
                 meta[i] = v+str(i)
 
-            label_img = torch.rand(100, 3, 10, 32)
+            label_img = torch.rand(100, 3, 32, 32)
             for i in range(100):
                 label_img[i]*=i/100.0
 
@@ -801,6 +872,10 @@ class SummaryWriter(object):
             writer.add_embedding(torch.randn(100, 5), label_img=label_img)
             writer.add_embedding(torch.randn(100, 5), metadata=meta)
         """
+
+        # programmer's note: This function has nothing to do with event files.
+        # The hard-coded projector_config.pbtxt is the only source for TensorBoard's
+        # current implementation. (as of Dec. 2019)
         from .x2num import make_np
         mat = make_np(mat)
         if global_step is None:
@@ -821,6 +896,7 @@ class SummaryWriter(object):
             make_tsv(metadata, save_path, metadata_header=metadata_header)
         if label_img is not None:
             assert mat.shape[0] == label_img.shape[0], '#images should equal with #data points'
+            assert label_img.shape[2] == label_img.shape[3], 'Image should be square, see tensorflow/tensorboard#670'
             make_sprite(label_img, save_path)
         assert mat.ndim == 2, 'mat should be 2D, where mat.size(0) is the number of data points'
         make_mat(mat, save_path)
@@ -1008,6 +1084,9 @@ class SummaryWriter(object):
         self._get_file_writer().add_summary(mesh(tag, vertices, colors, faces, config_dict), global_step, walltime)
 
     def close(self):
+        """Close the current SummaryWriter. This call flushes the unfinished write operation.
+        Use context manager (with statement) whenever it's possible.
+        """
         if self.all_writers is None:
             return  # ignore double close
         for writer in self.all_writers.values():
@@ -1016,6 +1095,9 @@ class SummaryWriter(object):
         self.file_writer = self.all_writers = None
 
     def flush(self):
+        """Force the data in memory to be flushed to disk. Use this call if tensorboard does not update reqularly.
+        Another way is to set the `flush_secs` when creating the SummaryWriter.
+        """
         if self.all_writers is None:
             return  # ignore double close
         for writer in self.all_writers.values():
